#include <sys/un.h>
#include <pthread.h>
#include "mep.h"

typedef struct md5list {
  char *id, *md5;
  struct md5list *next;
} md5list_t;

md5list_t *md5hash[10] = {
  NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL
};

pthread_mutex_t hashmutex = PTHREAD_MUTEX_INITIALIZER;

void hash_put(char *id, char *md5)
{
  md5list_t *temp;
  int total = 0;
  char *ch;

  for (ch = id; *ch != '\0'; ch++)
    total += (int) *ch;

  total %= 10;

  temp = (md5list_t *) malloc(sizeof(md5list_t));
  temp->id = id;
  temp->md5 = md5;
  temp->next = md5hash[total];
  md5hash[total] = temp;
}

char *hash_get(char *id)
{
  md5list_t **p, *temp;
  int total = 0;
  char *ch;

  for (ch = id; *ch != '\0'; ch++)
    total += (int) *ch;

  for (p = &md5hash[total % 10]; *p != NULL; p = &(*p)->next)
    if (strcmp(id, (*p)->id) == 0) {
      free((*p)->id);
      ch = (*p)->md5;
      temp = *p;
      *p = (*p)->next;
      free(temp);

      return ch;
    }

  return NULL;
}

void mep_verify(char *string)
{
  char *client_hash, *server_hash, *temp;

  string = strip(string, ' ');

  if ((client_hash = strchr(string, ' ')) != NULL) {
    *(client_hash++) = '\0';

    if (strncasecmp(client_hash, "MD5 ", 4) == 0) {
      client_hash += 3;
      while (*(++client_hash) == ' ');

      if ((temp = strstr(client_hash, "\r\n")) != NULL)
	*temp = '\0';

      pthread_mutex_lock(&hashmutex);

      server_hash = hash_get(string);

      pthread_mutex_unlock(&hashmutex);

      if (server_hash != NULL &&
	  strcmp(client_hash, server_hash) == 0) {
	sendout("OK\r\n");
	return;
      }
    }
  }

  sendout("ERRO\r\n");
}

void unixsocket(void)
{
  char *hash, *string;
  int desc, sock;
  struct sockaddr_un addr;
  socklen_t size = sizeof(struct sockaddr_un);

  addr.sun_family = AF_UNIX;
  strcpy(addr.sun_path, UNIX_SOCKET);

  if ((sock = socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
    fprintf(stderr, "Unable to get a socket: %s\n",
	    strerror(errno));
    exit(EXIT_FAILURE);
  }

  if (bind(sock, (struct sockaddr *) &addr, size) < 0) {
    fprintf(stderr, "Unable to bind socket: %s\n",
	    strerror(errno));
    exit(EXIT_FAILURE);
  }

  if (listen(sock, MAX_CONNECTIONS) < 0) {
    fprintf(stderr, "Unable to listen on socket: %s\n",
	    strerror(errno));
    exit(EXIT_FAILURE);
  }

  while (1) {
    if ((desc = accept(sock, (struct sockaddr *) &addr, &size))
	< 0) {
      fprintf(stderr, "Unable to accept connections: %s\n",
	      strerror(errno));
      exit(EXIT_FAILURE);
    }

    if ((string = getstring(desc, 0)) != NULL) {
      if (strncasecmp(string, "QUIT", 4) == 0) {
	close(sock);

	terminate();
      }

      if ((hash = strchr(string, ' ')) != NULL) {
	*(hash++) = '\0';

	pthread_mutex_lock(&hashmutex);

	hash_put(strdup(string), strdup(hash));

	pthread_mutex_unlock(&hashmutex);
      }

      free(string);
    }

    close(desc);
  }
}
